;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;This script used to plot e3sm diagnostics for TCs;;;;;;
;TC Basin definitions.(Knutson et. al.,2020,BAMS)
;North Atlantic: 0-90N, 265-360E
;Northeast Pacific: 0-90N, 180-265E
;Northwest Pacific: 0-90N, 100-180E
;North Indian: 0-90N, 30-100E
;South Indian: 90S-0, 20-135E
;Southwest Pacific: 90S-0, 135-295E
;South Atlantic(SA1+SA2): 90S-0, South America (295-360E) to Africa (0-20E)
;The North Atlantic-northeast Pacific boundary is on a
;diagonal tracing a path through Mexico and Central America.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
load "$NCARG_ROOT/lib/ncarg/nclscripts/csm/gsn_code.ncl"
load "$NCARG_ROOT/lib/ncarg/nclscripts/csm/gsn_csm.ncl"
load "$NCARG_ROOT/lib/ncarg/nclscripts/csm/contributed.ncl"
load "$NCARG_ROOT/lib/ncarg/nclscripts/esmf/ESMF_regridding.ncl"

begin

  ;;;;;;;;;;select the regions to calculate the rmse;;;;;;;;;;;;;
  datdir  = "/anvil/scratch/x-szhang3/tempest_240x120/ETCS/trk_psladj/"
  outdir  = "./fig_data/"
  ;;;;;;;;;;select the regions to calculate the rmse;;;;;;;;;;;;;
  Groups  = (/"CLIM30", "CLIM30ML","SSP245", "SSP245ML","SSP585", "SSP585ML" /)
  LabList = (/"CLIM",   "ML(CLIM)",\
              "SSP245 - CLIM", "ML(SSP245) - ML(CLIM)", \
              "SSP585 - CLIM", "ML(SSP585) - ML(CLIM)" /)
  ngrps   = dimsizes(Groups)
  ystr    = 1979
  yend    = 2014
  nyr     = yend - ystr +1
  figkey  = "etc_composite"

  Varnam  = (/"U850","V850", "T850", "IVT", "TUQ", "TVQ", "PSL"/)
  nvars   = dimsizes(Varnam)
  Varstr  = (/"U850","V850", "T850", "IVT", "TUQ", "TVQ", "PSL"/) 
  Varunt  = (/"m s~S~-1~N~", "m s~S~-1~N~", "K", "kg m~S~-1~N~ s~S~-1~N~", \
              "kg m~S~-1~N~ s~S~-1~N~", "kg m~S~-1~N~ s~S~-1~N~", "Pa"/)

  do i = 0,ngrps-1,1

   system("if ! test -d " + outdir +" ; then mkdir -p " + outdir + " ; fi")
   fno  = outdir+ Groups(i)+ "_etc_composite.nc"
   system("rm -f " + fno)
   print(" ")
   print("output : " + fno)
   print(" ")
   flo = addfile(fno,"c")

   do iv = 0,nvars-1,1
     fna   = new(nyr,string)
     vnm   = Varnam(iv)
     do iy = ystr,yend,1
       if(vnm.eq."TUQ".and.Groups(i).eq."ERA5") then 
         fna(iy-ystr) = systemfunc("ls -1 " + datdir+"/"+Groups(i)+"/"+Groups(i)+"*VIWVE*comp*"+iy+".nc")
       else if (vnm.eq."TVQ".and.Groups(i).eq."ERA5") then
         fna(iy-ystr) = systemfunc("ls -1 " + datdir+"/"+Groups(i)+"/"+Groups(i)+"*VIWVN*comp*"+iy+".nc")
       else 
         fna(iy-ystr) = systemfunc("ls -1 " + datdir+"/"+Groups(i)+"/"+Groups(i)+"*"+vnm+"*comp*"+iy+".nc")
       end if
       end if 
     end do 
     fla = addfiles(fna, "r")
     ListSetType (fla, "join")

     lat = fla[0]->y
     lon = fla[0]->x
     vdat = new((/nyr,dimsizes(lat),dimsizes(lon)/),typeof(lat))
     vdat!0 = "time"
     vdat!1 = "lat"
     vdat!2 = "lon"
     vdat&time = ispan(ystr,yend,1)
     vdat&lat  = lat 
     vdat&lon  = lon 
     if(vnm.eq."TUQ".and.Groups(i).eq."ERA5") then
       vdat(:,:,:) = (/fla[:]->VIWVE/)
     else if(vnm.eq."TVQ".and.Groups(i).eq."ERA5") then
       vdat(:,:,:) = (/fla[:]->VIWVN/)
     else 
       vdat(:,:,:) = (/fla[:]->$vnm$/)
     end if 
     end if 

     flo->$vnm$=vdat
     delete([/fna,vnm,fla,lat,lon,vdat/])

  end do 
end do 

end
