;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;This script used to plot e3sm diagnostics for TCs;;;;;;
;TC Basin definitions.(Knutson et. al.,2020,BAMS)
;North Atlantic: 0-90N, 265-360E
;Northeast Pacific: 0-90N, 180-265E
;Northwest Pacific: 0-90N, 100-180E
;North Indian: 0-90N, 30-100E
;South Indian: 90S-0, 20-135E
;Southwest Pacific: 90S-0, 135-295E
;South Atlantic(SA1+SA2): 90S-0, South America (295-360E) to Africa (0-20E)
;The North Atlantic-northeast Pacific boundary is on a
;diagonal tracing a path through Mexico and Central America.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
load "$NCARG_ROOT/lib/ncarg/nclscripts/csm/gsn_code.ncl"
load "$NCARG_ROOT/lib/ncarg/nclscripts/csm/gsn_csm.ncl"
load "$NCARG_ROOT/lib/ncarg/nclscripts/csm/contributed.ncl"
load "$NCARG_ROOT/lib/ncarg/nclscripts/esmf/ESMF_regridding.ncl"

begin

  ;;;;;;;;;;select the regions to calculate the rmse;;;;;;;;;;;;;
  datdir  = "./fig_data/"
  ;;;;;;;;;;select the regions to calculate the rmse;;;;;;;;;;;;;
  Groups  = (/"CLIM30", "CLIM30ML","SSP245", "SSP245ML","SSP585", "SSP585ML" /)
  LabList = (/"CLIM",   "ML(CLIM)",\
              "SSP245 - CLIM", "ML(SSP245) - ML(CLIM)", \
              "SSP585 - CLIM", "ML(SSP585) - ML(CLIM)" /)
  ngrps   = dimsizes(Groups)

  Figstr  = (/"(a)", "(b)", "(a)", "(b)", "(c)", "(d)"/)
  ystr    = 1979
  yend    = 2014
  nyr     = yend - ystr +1
  figkey  = "etc_future_composite"

  Varnam  = (/"U850","V850", "T850", "IVT"/)
  nvars   = dimsizes(Varnam)
  Varplt  = "var850"

  do i = 0,ngrps-1,1

   do iv = 0,nvars-1,1
     fna = datdir+ Groups(i)+ "_etc_composite.nc"
     vnm = Varnam(iv)
     fla = addfiles(fna, "r")
     ListSetType (fla, "join")
     lat = fla[0]->lat
     lon = fla[0]->lon
     if(iv.eq.0.and.i.eq.0) then 
       vall = new((/ngrps,nvars,nyr,dimsizes(lat),dimsizes(lon)/),typeof(lat))
       vall!0 = "group"
       vall!1 = "var"
       vall!2 = "time"
       vall!3 = "lat"
       vall!4 = "lon"
       vall&time = ispan(ystr,yend,1)
       vall&lat  = lat 
       vall&lon  = lon 
     end if 
     vall(i,iv,:,:,:) = (/fla[:]->$vnm$/)
     delete([/fna,vnm,fla,lat,lon/])
   end do 
  end do 

  npts   = 8 
  lat    = conform(vall(0,:,:,:,:),vall&lat,(/2/))
  lon    = conform(vall(0,:,:,:,:),vall&lon,(/3/))
  gcdist = gc_latlon(0,0,lat,lon, 2,2)
  lmsk   = vall(0,:,:,:,:)
  lmsk   = 0
  lmsk   = where(abs(gcdist).gt.24.8,0.0,1.0)
  delete([/lat,lon,npts/])

  do i = 2,ngrps-1,1

   if(Groups(i).eq."SSP245".or.Groups(i).eq."SSP585") then
     indref = ind(Groups.eq."CLIM30")
   else
     indref = ind(Groups.eq."CLIM30ML")
   end if
   vref = vall(indref,:,:,:,:)
   vdat = vall(i,:,:,:,:) 

   vdat = smth9_Wrap(vdat, 0.50, -0.25, False)
   vref = smth9_Wrap(vref, 0.50, -0.25, False)
   vdat(2,:,:,:) = where(abs(gcdist(2,:,:,:)).gt.24.2,vdat@_FillValue,vdat(2,:,:,:))
   vref(2,:,:,:) = where(abs(gcdist(2,:,:,:)).gt.24.2,vref@_FillValue,vref(2,:,:,:))

   xAve = dim_avg_n(vref(3,:,:,:),0)
   yAve = dim_avg_n(vdat(3,:,:,:),0)

   xVar = dim_variance_n(vref(3,:,:,:),0)
   yVar = dim_variance_n(vdat(3,:,:,:),0)

   xN   = nyr
   yN   = nyr

   sigr  = 0.05                                   ; critical sig lvl for r
   iflag = False                        ; population variance similar
   prob  = xAve
   prob  = (/ttest(xAve,xVar,xN, yAve,yVar,yN, iflag, False)/)
   prob  = where(prob.lt.sigr,prob,prob@_FillValue )
   prob  = where(lmsk(0,3,:,:).eq.1,prob,prob@_FillValue)
   prob!0 = "lat"
   prob!1 = "lon"
   prob&lat = vdat&lat 
   prob&lon = vdat&lon
   delete([/xAve,yAve,xVar,yVar,sigr,xN,yN,iflag/])
   plot_ttest = True 
   vdat = vdat - vref
   vdat = where(lmsk.eq.0.0,vdat@_FillValue,vdat)
   varm = dim_avg_n_Wrap(vdat,1) 

   ;----------------------------------------------------------------------
   ; Plotting section
   ;----------------------------------------------------------------------

   if(i.eq.2)then
    ;;;;;;;open a wks for plot;;;;;;;;;;;;;;;;;;;
    wtype = "pdf"
    wtype@wkPaperSize  = "A4"
   ;wtype@wkOrientation   = "landscape" ;;pictures displayed along the length
 
    plotname = Varplt + "_" + figkey 
    wks1     = gsn_open_wks(wtype,plotname)
    plot     = new (ngrps, graphic)

    print(" ")
    print("........................... ")
    print("plot : " + plotname+".pdf")
    print("........................... ")

   end if 

   ;;;set up the plot information;;;;
   lndcolor   = "Black" ;"Grey50"
   cnlevels  := fspan(256,272,9) ;fspan(960,1008,13) 
   cnlevel2  := (/-1.6,-1.2,-0.8,-0.4,-0.1,0.1,0.4,0.8,1.2,1.6/)*10.0
   cnlevel2  := fspan(0,30,11);(/0,1,10,20,30,40/)
   cmapx = read_colormap_file("percent_11lev")
   cmapx(0,:) = 0 

;---Resources to share between both plots
   res                = True     ; Plot modes desired.
   res@gsnDraw        = False
   res@gsnFrame       = False
   res@gsnMaximize    = False     ; Maximize plot
   res@tmXTOn         = False
   res@tmYROn         = False

   BorderThick = 1.0
   res@tmBorderThicknessF    = BorderThick
   res@tmXBMajorThicknessF   = BorderThick
   res@tmXBMinorThicknessF   = BorderThick*0.5
   res@tmYLMajorThicknessF   = BorderThick
   res@tmYLMinorThicknessF   = BorderThick*0.5
   res@tmXTMajorThicknessF   = BorderThick
   res@tmXTMinorThicknessF   = BorderThick*0.5
   res@tmYRMajorThicknessF   = BorderThick
   res@tmYRMinorThicknessF   = BorderThick*0.5

   FontHeightF = 0.028
   res@tiMainFontHeightF    = FontHeightF
   res@tmYLLabelFontHeightF = FontHeightF
   res@tmXBLabelFontHeightF = FontHeightF

   mpres  = res
   resx   = res
   resx1  = res

   res@gsnStringFontHeightF  = FontHeightF*1.0
   res@lbLabelFontHeightF    = FontHeightF*0.9
   res@lbTitleFontHeightF    = FontHeightF*0.8
   res@tiXAxisFontHeightF    = FontHeightF*1.0
   res@tiYAxisFontHeightF    = FontHeightF*1.0

   res@cnFillOn              = True     ; color plot desired
   res@cnLinesOn             = False    ; turn off contour lines
   res@cnLineLabelsOn        = False    ; turn off contour labels
  ;res@cnFillMode            = "RasterFill"   ; turn raster on
   res@cnMissingValFillColor = "White"

   res@lbLabelBarOn             = False   ; Will turn on in panel later
   res@pmLabelBarParallelPosF   = 0.50 ;61
   res@pmLabelBarOrthogonalPosF = 0.05
   res@lbOrientation            = "vertical"  ; vertical label bar
   res@pmLabelBarHeightF        = 0.65
   res@pmLabelBarWidthF         = 0.08
   res@lbTitlePosition          = "Right"         ; title location
   res@lbTitleDirection         = "Across"        ; letter angle
   res@lbTitleAngleF            = 90.          ; title angle
   res@lbBoxLinesOn             = True

   res@gsnCenterString    = ""
   res@gsnLeftString      = ""
   res@gsnRightString     = "" 
   res@tiMainString       = "" 

   res@trYMinF = -25
   res@trYMaxF =  25
   res@trXMinF = -25
   res@trXMaxF =  25
   res@trYReverse = True 

   res@tmXBMode          = "Explicit"              ; explicit labels
   res@tmXBValues        = ispan(-30,30,10)        ; location of labels 
   res@tmXBLabels        = ispan(-30,30,10)
   res@tmXBMinorValues   = ispan(-30,30,5)
   res@tiXAxisString     = "X (Degree)"       ; Put label on X axis

   res@tmYLMode          = "Explicit"              ; explicit labels
   res@tmYLValues        = ispan(-30,30,10)
   res@tmYLLabels        = ispan(-30,30,10)
   res@tmYLMinorValues   = ispan(-30,30,5)
   res@tiYAxisString     = "Y (Degree)"       ; Put label on X axis

   res@tmXMajorGrid                = False         ; implement x grid 
   res@tmXMajorGridThicknessF      = 1.0           ; 2.0 is default
   res@tmXMajorGridLineDashPattern = 2             ; select short dash lines
  
   res@tmYMajorGrid                = False         ; implement y grid
   res@tmYMajorGridThicknessF      = 1.0           ; 2.0 is default
   res@tmYMajorGridLineDashPattern = 2             ; select short dash lines

  ;---Resources for plotting original data
   res1=res
   res1@cnLevelSelectionMode        = "ExplicitLevels"  
   res1@cnSmoothingOn               = True
   res1@cnLinesOn                   = True 
   res1@cnLineLabelsOn              = False  
   res1@cnInfoLabelOn               = False
   res1@cnLineLabelDensityF         = 2.0
   res1@cnLineLabelFontHeightF      = 0.020
   res1@gsnContourLineThicknessesScale = 1.0
   res1@cnLineThicknessF            = 0.001
   res1@cnLineColor                 = "grey25"
   res1@gsnContourZeroLineThicknessF= 0.2
   res1@gsnContourNegLineDashPattern= 0

   res1@gsnLeftString               = Figstr(i) + "  " +LabList(i)
   res1@gsnLeftStringOrthogonalPosF = 0.04
   res1@gsnRightString              = ""
   res1@gsnRightStringOrthogonalPosF= 0.04

   resV=True
   resV@gsnFrame                = False
   resV@gsnDraw                 = False
   resV@vcLineArrowColor        = "Grey25"
   resV@vcRefLengthF            = 0.040  ; reference length    
   resV@vcGlyphStyle            = "LineArrow";"CurlyVector"  ; curly vectors
   resV@vcMinDistanceF          = 0.03  ; minimum distance
   resV@vcLineArrowThicknessF   = 1.5 ;arrow thickness
   resV@vcRefAnnoFontHeightF    = FontHeightF*0.8 ; label size
   resV@vcRefAnnoOn             = True ; turn label on
   resV@vcRefAnnoString2On      = False ; turn default string off
   resV@vcRefAnnoOrthogonalPosF = -0.24 ; label position
   resV@vcRefAnnoParallelPosF   = 0.99
   resV@vcRefAnnoPerimSpaceF    = 0.33 ; label box space 
   resV@vcRefAnnoArrowSpaceF    = 0.7


   res1@cnLevels                := cnlevel2
   res1@cnFillPalette           := cmapx ;"precip3_16lev" ;"CBR_coldhot" ;"hotcolr_19lev" 
   res1@lbTitleString            = ""
   plot(i) = gsn_csm_contour(wks1,varm(3,:,:),res1)

   res2 = res1
   res2@gsnCenterString    = ""
   res2@gsnLeftString      = ""
   res2@gsnRightString     = ""
   res2@tiMainString       = ""
   res2@cnFillOn  = False
   res2@cnLinesOn = True
   res2@cnInfoLabelOn             = False
   res2@cnLineThicknessF          = 1.5
   res2@cnLineColor               = "Blue"
   res2@cnLineLabelsOn            = True
   res2@cnLineLabelFontHeightF    = FontHeightF*0.9
   res2@cnLineLabelFontThicknessF = res2@cnLineThicknessF
   res2@cnLineLabelFontColor      = "black";res2@cnLineColor
   res2@cnLineLabelBackgroundColor= "grey90"
   if(Groups(i).eq."SSP245".or.Groups(i).eq."SSP245ML") then
     res2@cnLevels              := fspan(0.4,1.6,13)
   else
     res2@cnLevels              := fspan(0.8,2.4,17)
   end if 
   plott  = gsn_csm_contour(wks1,varm(2,:,:),res2)

   refmag = 0.3
   resV@vcMinDistanceF           = 0.025  ; minimum distance
   resV@vcRefMagnitudeF          = refmag      ; reference magnitude
   resV@vcRefAnnoString1         = refmag +"m/s"
   plotu   = gsn_csm_vector(wks1,varm(0,:,:),varm(1,:,:),resV) ; arrows drawing

   if(plot_ttest) then
     res21 = res
     res21@gsnLeftString                = ""
     res21@gsnRightString               = ""
     res21@gsnCenterString              = ""
     res21@cnMissingValFillColor        = "Transparent"
     res21@cnFillOn                     = True
     res21@cnLinesOn                    = False
     res21@cnLineLabelsOn               = False       ; turn off contour labels
     res21@cnInfoLabelOn                = False
     res21@cnMonoFillColor              = True        ; default color is fground [black]
     res21@cnMonoFillPattern            = True
     res21@cnMonoFillScale              = True
     res21@cnFillPattern                = 17
     res21@cnFillScaleF                 = 0.9
     res21@cnFillDotSizeF               = 0.001
     res21@lbLabelBarOn                 = False
     res21@cnFillColor                  = "Grey60"
     plot_cont2 = gsn_csm_contour(wks1,prob,res21)
     overlay(plot(i),plot_cont2)
     delete([/res21,prob/])
   end if

   overlay(plot(i),plott) ; overlaying
   overlay(plot(i),plotu) ; overlaying

   ;************************************************
   ; add error bars
   ;************************************************  
   polyres                   = True                       ; marker resources
   polyres@gsMarkerIndex     = 5                          ; index for circle
   polyres@gsMarkerSizeF     = .008                        ; size 
   polyres@gsMarkerThicknessF= 2.0
   polyres@gsMarkerColor     = "white"
   if(.not.isvar("dum")) then
     dum = new(ngrps,graphic)
   end if 
   dum(i) = gsn_add_polymarker(wks1,plot(i),(/0,0/),(/0,0/),polyres)  

   delete([/res,res1,mpres/])

 end do ; end of experiment group loop

;---Draw both plots in a panel
;***************************************
; panel first set of plots
;***************************************
  pres1                          = True
  pres1@gsnFrame                 = True      ; don't advance frame yet
  pres1@gsnMaximize              = False
  pres1@gsnPanelLabelBar         = True
  pres1@pmLabelBarParallelPosF   = 0.025
  pres1@pmLabelBarOrthogonalPosF = -0.03
  pres1@lbOrientation            = "horizontal"  ; vertical label bar
  pres1@pmLabelBarHeightF        = 0.05 ;0.21
  pres1@pmLabelBarWidthF         = 0.68 ;0.05
  pres1@lbLabelFontHeightF       = FontHeightF * 0.45
  pres1@lbTitleFontHeightF       = FontHeightF * 0.5
  pres1@lbTitleString            = "Response (~F33~D~N~~F~) of IVT (kg m~S~-1~N~ s~S~-1~N~)"
 ;pres1@lbTitlePosition          = "Left";"Right"         ; title location
 ;pres1@lbTitleDirection         = "Across"        ; letter angle
 ;pres1@lbTitleAngleF            = 90.          ; title angle
 ;pres1@lbBoxLinesOn             = True
 ;pres1@lbBoxSeparatorLinesOn    = False
 ;pres1@lbLabelStride            = 15

  pres1@gsnPanelMainString       = " "
  pres1@gsnPanelMainFontColor    = "Black"
  pres1@gsnPanelMainFontHeightF  = 0.028
  pres1@gsnPanelYWhiteSpacePercent = 2
  pres1@gsnPanelXWhiteSpacePercent = 2
  gsn_panel(wks1,(/plot(2:)/),(/2,2/),pres1)

end
